unit formMain;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.StdCtrls,
  FMX.Controls.Presentation, FMX.Viewport3D, IPPeerClient, IPPeerServer,
  System.Tether.Manager, System.Tether.AppProfile, uScene;

type
  TfrmMain = class(TForm)
    viewport: TViewport3D;
    pnlControl: TPanel;
    btnFullScreen: TSpeedButton;
    TetheringManager: TTetheringManager;
    TetheringAppProfile: TTetheringAppProfile;
    procedure btnFullScreenClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; var KeyChar: Char; Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure viewportPaint(Sender: TObject; Canvas: TCanvas; const ARect: TRectF);
    procedure TetheringAppProfileResourceReceived(const Sender: TObject;
      const AResource: TRemoteResource);
    procedure TetheringAppProfileAfterConnectProfile(const Sender: TObject;
      const AProfileInfo: TTetheringProfileInfo);
    procedure TetheringManagerUnPairManager(const Sender: TObject;
      const AManagerInfo: TTetheringManagerInfo);
  private
    fFullScreenMode: boolean;
    fWindowWidth: integer;
    fWindowHeight: integer;
    fWindowTop: integer;
    fWindowLeft: integer;
    fScene: TScene;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.fmx}

procedure TfrmMain.btnFullScreenClick(Sender: TObject);
begin
  if fFullScreenMode then begin
    exit;
  end;
  fWindowWidth := frmMain.Width;
  fWindowHeight := frmMain.Height;
  fWindowLeft := frmMain.Left;
  fWindowTop := frmMain.Top;
  pnlControl.Visible := False;
  frmMain.BorderStyle := TFmxFormBorderStyle.None;
  frmMain.WindowState := TWindowState.wsMaximized;
  fFullScreenMode := True;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  fFullScreenMode := False;
  fScene := TScene.Create;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  if assigned(fScene) then begin
    fScene.DisposeOf;
    fScene := nil;
  end;
end;

procedure TfrmMain.FormKeyDown(Sender: TObject; var Key: Word; var KeyChar: Char; Shift: TShiftState);
begin
  if Key=vkEscape then begin
    if not fFullScreenMode then begin
      exit;
    end;
    pnlControl.Visible := True;
    frmMain.BorderStyle := TFmxFormBorderStyle.Sizeable;
    frmMain.WindowState := TWindowState.wsNormal;
    frmMain.Width := fWindowWidth;
    frmMain.Height := fWindowHeight;
    frmMain.Top := fWindowTop;
    frmMain.Left := fWindowLeft;
    fFullScreenMode := False;
  end;
end;

procedure TfrmMain.TetheringAppProfileAfterConnectProfile(const Sender: TObject; const AProfileInfo: TTetheringProfileInfo);
begin
  TetheringAppProfile.SendString(TetheringManager.RemoteProfiles.First, 'DesktopName', GetEnvironmentVariable('COMPUTERNAME'));
end;

procedure TfrmMain.TetheringAppProfileResourceReceived(const Sender: TObject; const AResource: TRemoteResource);
var
  NewAction: TSceneAction;
begin
  if not (Aresource.Hint='SceneUpdate') then begin
    exit;
  end;
  AResource.Value.AsStream.Position := 0;
  if AResource.Value.AsStream.Size>0 then begin
    TScene.ActionFromStream(NewAction,AResource.Value.AsStream);
    fScene.AddAction(NewAction);
    viewport.InvalidateRect(viewport.ClipRect);
  end;
end;

procedure TfrmMain.TetheringManagerUnPairManager(const Sender: TObject; const AManagerInfo: TTetheringManagerInfo);
begin
  fScene.Clear;
  viewport.InvalidateRect(viewport.ClipRect);
end;

procedure TfrmMain.viewportPaint(Sender: TObject; Canvas: TCanvas; const ARect: TRectF);
begin
  if not assigned(fScene) then begin
    exit;
  end;
  fScene.Render(Canvas,ARect);
end;

end.
