unit formMain;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.StdCtrls,
  FMX.Controls.Presentation, FMX.Viewport3D, IPPeerClient, IPPeerServer,
  FMX.MultiView, System.Tether.Manager, System.Tether.AppProfile, FMX.ListBox,
  uScene, FMX.Colors, FMX.Layouts, FMX.Edit, FMX.EditBox, FMX.SpinBox;

type
  TfrmMain = class(TForm)
    TetheringAppProfile: TTetheringAppProfile;
    TetheringManager: TTetheringManager;
    pnlTBar: TPanel;
    MultiView: TMultiView;
    btnDrawer: TSpeedButton;
    viewport: TViewport3D;
    pnlConnection: TPanel;
    btnScan: TSpeedButton;
    lblConnection: TLabel;
    StyleBook1: TStyleBook;
    btnClear: TSpeedButton;
    btnUndo: TSpeedButton;
    pnlDrawColor: TPanel;
    cboBackColor: TColorComboBox;
    Label1: TLabel;
    Panel1: TPanel;
    Label2: TLabel;
    spedPen: TSpinBox;
    colPen: TColorComboBox;
    GridLayout1: TGridLayout;
    ColorButton1: TColorButton;
    ColorButton2: TColorButton;
    ColorButton3: TColorButton;
    ColorButton4: TColorButton;
    ColorButton5: TColorButton;
    ColorButton6: TColorButton;
    ColorButton7: TColorButton;
    ColorButton8: TColorButton;
    ColorButton9: TColorButton;
    ColorButton10: TColorButton;
    ColorButton11: TColorButton;
    ColorButton12: TColorButton;
    ColorButton13: TColorButton;
    ColorButton14: TColorButton;
    ColorButton15: TColorButton;
    ColorButton16: TColorButton;
    trkButtonScale: TTrackBar;
    procedure btnScanClick(Sender: TObject);
    procedure TetheringManagerPairedToRemote(const Sender: TObject; const AManagerInfo: TTetheringManagerInfo);
    procedure TetheringAppProfileResourceReceived(const Sender: TObject; const AResource: TRemoteResource);
    procedure TetheringManagerEndAutoConnect(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TetheringAppProfileDisconnect(const Sender: TObject; const AProfileInfo: TTetheringProfileInfo);
    procedure viewportMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Single);
    procedure viewportMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Single);
    procedure viewportMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Single);
    procedure viewportMouseLeave(Sender: TObject);
    procedure viewportPaint(Sender: TObject; Canvas: TCanvas; const ARect: TRectF);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnUndoClick(Sender: TObject);
    procedure cboBackColorChange(Sender: TObject);
    procedure colPenChange(Sender: TObject);
    procedure TetheringManagerUnPairManager(const Sender: TObject; const AManagerInfo: TTetheringManagerInfo);
    procedure btnClearClick(Sender: TObject);
    procedure ColorButton1Click(Sender: TObject);
    procedure spedPenChange(Sender: TObject);
    procedure trkButtonScaleChange(Sender: TObject);
  private
    fConnected: boolean;
    fScene: TScene;
    procedure HandleNewAction(NewAction: TSceneAction);
    procedure EnableControls;
    procedure DisableControls;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.fmx}

procedure TfrmMain.EnableControls;
begin
  ColorButton1.Enabled := True;
  ColorButton2.Enabled := True;
  ColorButton3.Enabled := True;
  ColorButton4.Enabled := True;
  ColorButton5.Enabled := True;
  ColorButton6.Enabled := True;
  ColorButton7.Enabled := True;
  ColorButton8.Enabled := True;
  ColorButton9.Enabled := True;
  ColorButton10.Enabled := True;
  ColorButton11.Enabled := True;
  ColorButton12.Enabled := True;
  ColorButton13.Enabled := True;
  ColorButton14.Enabled := True;
  ColorButton15.Enabled := True;
  ColorButton16.Enabled := True;
  btnUndo.Enabled := True;
  btnClear.Enabled := True;
  colPen.Enabled := True;
  cboBackColor.Enabled := True;
  spedPen.Enabled := True;
end;

procedure TfrmMain.DisableControls;
begin
  ColorButton1.Enabled := False;
  ColorButton2.Enabled := False;
  ColorButton3.Enabled := False;
  ColorButton4.Enabled := False;
  ColorButton5.Enabled := False;
  ColorButton6.Enabled := False;
  ColorButton7.Enabled := False;
  ColorButton8.Enabled := False;
  ColorButton9.Enabled := False;
  ColorButton10.Enabled := False;
  ColorButton11.Enabled := False;
  ColorButton12.Enabled := False;
  ColorButton13.Enabled := False;
  ColorButton14.Enabled := False;
  ColorButton15.Enabled := False;
  ColorButton16.Enabled := False;
  btnUndo.Enabled := False;
  btnClear.Enabled := False;
  colPen.Enabled := False;
  cboBackColor.Enabled := False;
  spedPen.Enabled := False;
end;

procedure TfrmMain.HandleNewAction( NewAction: TSceneAction );
var
  MemoryStream: TMemoryStream;
begin
  if not fConnected then begin
    exit;
  end;
  MemoryStream := TMemoryStream.Create;
  try
    TScene.ActionToStream( NewAction, MemoryStream );
    MemoryStream.Position := 0;
    TetheringAppProfile.SendStream(TetheringManager.RemoteProfiles.First,'SceneUpdate',MemoryStream);
  finally
    MemoryStream.DisposeOf;
  end;
end;

procedure TfrmMain.spedPenChange(Sender: TObject);
var
  NewAction: TSceneAction;
begin
  NewAction.PointCount := 0;
  NewAction.Action := TSceneActionType.saPenWidth;
  NewAction.Data := Trunc(spedPen.Value);
  fScene.AddAction(NewAction);
  viewport.InvalidateRect(viewport.ClipRect);
end;

procedure TfrmMain.btnClearClick(Sender: TObject);
var
  NewAction: TSceneAction;
begin
  NewAction.Action := TSceneActionType.saClear;
  NewAction.PointCount := 0;
  fScene.AddAction(NewAction);
  NewAction.Action := TSceneActionType.saBackColor;
  NewAction.Data := uint64(cboBackColor.Color);
  fScene.AddAction(NewAction);
  NewAction.Action := TSceneActionType.saStrokeColor;
  NewAction.Data := uint64(colPen.Color);
  fScene.AddAction(NewAction);
  NewAction.Action := TSceneActionType.saPenWidth;
  NewAction.Data := Trunc(spedPen.Value);
  fScene.AddAction(NewAction);
  viewport.InvalidateRect(viewport.ClipRect);
end;

procedure TfrmMain.btnScanClick(Sender: TObject);
begin
  if fConnected then begin
    TetheringManager.UnPairManager(TetheringManager.RemoteManagers.First);
    fConnected := False;
    DisableControls;
  end;
  TetheringManager.AutoConnect(4000);
  lblConnection.Text := 'Scanning...';
  lblConnection.TextSettings.FontColor := TAlphaColorRec.Gold;
end;

procedure TfrmMain.btnUndoClick(Sender: TObject);
var
  NewAction: TSceneAction;
begin
  NewAction.Action := TSceneActionType.saUndo;
  NewAction.Data := 0;
  NewAction.PointCount := 0;
  fScene.AddAction(NewAction);
  viewport.InvalidateRect(viewport.cliprect);
end;

procedure TfrmMain.cboBackColorChange(Sender: TObject);
var
  NewAction: TSceneAction;
begin
  NewAction.PointCount := 0;
  NewAction.Action := saBackColor;
  NewAction.Data := uint64(cboBackColor.Color);
  fScene.AddAction(NewAction);
  viewport.InvalidateRect(viewport.ClipRect);
end;

procedure TfrmMain.ColorButton1Click(Sender: TObject);
begin
  TColorButton(Sender).Opacity := 1;
  colPen.Color := TColorButton(Sender).Color;
  colPen.OnChange(colPen);
end;

procedure TfrmMain.colPenChange(Sender: TObject);
var
  NewAction: TSceneAction;
begin
  NewAction.Action := TSceneActionType.saStrokeColor;
  NewAction.Data := uint64(colPen.Color);
  NewAction.PointCount := 0;
  fScene.AddAction(NewAction);
  viewport.InvalidateRect(viewport.ClipRect);
end;

procedure TfrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := True;
  if fConnected then begin
    TetheringManager.UnPairManager(TetheringManager.RemoteManagers.First);
  end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  fConnected := False;
  DisableControls;
  fScene := TScene.Create;
  fScene.OnAction := HandleNewAction;
  MultiView.Mode := TMultiViewMode.Drawer;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  if assigned(fScene) then begin
    fScene.DisposeOf;
    fScene := nil;
  end;
end;

procedure TfrmMain.TetheringAppProfileDisconnect(const Sender: TObject; const AProfileInfo: TTetheringProfileInfo);
begin
  fConnected := False;
  DisableControls;
  lblConnection.Text := 'Not Connected';
  lblConnection.TextSettings.FontColor := TAlphaColorRec.Crimson;
end;

procedure TfrmMain.TetheringAppProfileResourceReceived(const Sender: TObject; const AResource: TRemoteResource);
begin
  if AResource.Hint='DesktopName' then begin
    lblConnection.Text := Format('Connected : %s',[AResource.Value.AsString]);
    lblConnection.TextSettings.FontColor := TAlphaColorRec.Green;
  end;
end;

procedure TfrmMain.TetheringManagerEndAutoConnect(Sender: TObject);
begin
  if not fConnected then begin
    lblConnection.Text := 'Not Connected';
    lblConnection.TextSettings.FontColor := TAlphaColorRec.Crimson;
  end;
end;

procedure TfrmMain.TetheringManagerPairedToRemote(const Sender: TObject; const AManagerInfo: TTetheringManagerInfo);
begin
  fConnected := True;
  EnableControls;
  lblConnection.Text := Format('Connected : %s %s', [AManagerInfo.ManagerIdentifier, AManagerInfo.ManagerName]);
  lblConnection.TextSettings.FontColor := TAlphaColorRec.Green;
end;

procedure TfrmMain.TetheringManagerUnPairManager(const Sender: TObject; const AManagerInfo: TTetheringManagerInfo);
begin
  fScene.Clear;
  viewport.InvalidateRect(viewport.ClipRect);
end;

procedure TfrmMain.trkButtonScaleChange(Sender: TObject);
begin
  GridLayout1.ItemWidth := TTrackBar(Sender).Value;
  GridLayout1.ItemHeight := TTrackBar(Sender).Value;
end;

procedure TfrmMain.viewportMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Single);
begin
  if not fConnected then begin
    exit;
  end;
  fScene.BeginNewAction(TSceneActionType.saFreehand,X,Y);
end;

procedure TfrmMain.viewportMouseLeave(Sender: TObject);
begin
  if not fConnected then begin
    exit;
  end;
  fScene.EndAction;
end;

procedure TfrmMain.viewportMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Single);
begin
  if not fConnected then begin
    exit;
  end;
  fScene.UpdateAction(X,Y);
  viewport.InvalidateRect(viewport.ClipRect);
end;

procedure TfrmMain.viewportMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Single);
begin
  if not fConnected then begin
    exit;
  end;
  fScene.EndAction;
end;

procedure TfrmMain.viewportPaint(Sender: TObject; Canvas: TCanvas; const ARect: TRectF);
begin
  if not assigned(fScene) then begin
    exit;
  end;
  fScene.Render(Canvas,ARect);
end;

end.
